program main
  use mndr


  call set_parameter
  call read_fln
  call cal_critical_shear_stress

  do n = 1, nend

     call read_array
     call set_array

     call read_scalar_csv

     call calculate_spatial_slope(u_cal  , dudx_cal  , dudy_cal  , dudr_cal)
     call calculate_spatial_slope(v_cal  , dvdx_cal  , dvdy_cal  , dvdr_cal)
     call calculate_spatial_slope(bl     , ibi       , ibj       , ib)

     call calculate_spatial_curvature(bl , cbi       , cbj       , cb)
     call calculate_energy_slope(nb, dep_cal, u_cal, v_cal, iei, iej, ie)

     call calculate_spatial_slope(iei    , diexdx_cal, diexdy_cal, diexdr_cal)
     call calculate_spatial_slope(iej    , dieydx_cal, dieydy_cal, dieydr_cal)
     call calculate_spatial_slope(ie     , diedx_cal , diedy_cal , diedr_cal)
     call calculate_spatial_slope(vel_cal, dveldx_cal, dveldy_cal, dveldr_cal)

     call calculate_advection_diffusion_equation_of_bed_level(nb, dep_cal, iei, iej, ie, ibi, ibj,  &
          cbi, cbj, u_cal, v_cal, vel_cal, diexdx_cal, diexdy_cal, dieydx_cal, dieydy_cal,      &
          diedx_cal, diedy_cal, dudx_cal, dudy_cal, dvdx_cal, dvdy_cal, dveldx_cal, dveldy_cal, &
          Mx, My, Mxy, fx, fy, fx2, fy2, Df, dz_cal)

      call output_scalar_result

     call deallocate

  end do

end program main




subroutine set_parameter
  use mndr

  case = '20210715'   !  You should select directory name in 01_Input_csv
  dm   = 0.00076      !  average diameter of soil particles [m]
  dt   = 60         !  Delta t for calculation of bedload variation [s]

  pi   = acos(-1.0d0) !  pi
  g    = 9.81         !  gravitational accelaration [m/s^2]
  s    = 1.65         !  underwater specific gravity of soil particles

end subroutine set_parameter



subroutine set_array
  use mndr

  allocate(x(iend,jend)         , y(iend,jend)         , nb(iend,jend))
  allocate(bl(iend,jend))

  allocate(dep_cal(iend,jend)   , wl_cal(iend,jend))
  allocate(u_cal(iend,jend)     , v_cal(iend,jend), vel_cal(iend,jend))


  allocate(dudx_cal(iend,jend)  , dudy_cal(iend,jend)  , dudr_cal(iend,jend))
  allocate(dvdx_cal(iend,jend)  , dvdy_cal(iend,jend)  , dvdr_cal(iend,jend))
  allocate(ibi(iend,jend)       , ibj(iend,jend)       , ib(iend,jend))
  allocate(cbj(iend,jend)       , cbi(iend,jend)       , cb(iend,jend))
  allocate(iei(iend,jend)       , iej(iend,jend)       , ie(iend,jend))
  allocate(diexdx_cal(iend,jend), diexdy_cal(iend,jend), diexdr_cal(iend,jend))
  allocate(dieydx_cal(iend,jend), dieydy_cal(iend,jend), dieydr_cal(iend,jend))
  allocate(diedx_cal(iend,jend) , diedy_cal(iend,jend) , diedr_cal(iend,jend))
  allocate(dveldx_cal(iend,jend), dveldy_cal(iend,jend), dveldr_cal(iend,jend))

  allocate(Mx(iend,jend)        , My(iend,jend)        , Mxy(iend,jend))
  allocate(fx(iend,jend)        , fy(iend,jend))
  allocate(fx2(iend,jend)       , fy2(iend,jend))
  allocate(Df(iend,jend)        , Pe(iend,jend))
  allocate(dz_cal(iend,jend))

end subroutine set_array




subroutine deallocate
  use mndr

  deallocate(x         , y    , nb)
  deallocate(bl)

  deallocate(dep_cal   , wl_cal)
  deallocate(u_cal     , v_cal     , vel_cal)

  deallocate(dudx_cal  , dudy_cal  , dudr_cal)
  deallocate(dvdx_cal  , dvdy_cal  , dvdr_cal)
  deallocate(ibi       , ibj       , ib)
  deallocate(cbj       , cbi       , cb)
  deallocate(iei       , iej       , ie)
  deallocate(diexdx_cal, diexdy_cal, diexdr_cal)
  deallocate(dieydx_cal, dieydy_cal, dieydr_cal)
  deallocate(diedx_cal , diedy_cal , diedr_cal)
  deallocate(dveldx_cal, dveldy_cal, dveldr_cal)   

  deallocate(Mx        , My        , Mxy)
  deallocate(fx        , fy)
  deallocate(fx2       , fy2)
  deallocate(Df        , Pe)
  deallocate(dz_cal)

end subroutine deallocate



subroutine read_fln
  use mndr  

  call system('ls -1 ../01_Input_csv/'//trim(case)//'/ > ./file.txt')

  open(100,file = 'file.txt')  

  ! ----  Counting number of files (nend) ----
  n = 0
  do
     n = n + 1
     read(100, *, end = 998)
  enddo
998 nend = n - 1
  close(100)

  allocate(fln(nend),fln2(nend))
  allocate(time1(nend))
  allocate(time2(nend))

  open(100,file = 'file.txt')  
  read(100,*)(fln(n), n = 1, nend)
  close(100)

  open(100,file = 'file.txt')  
  read(100,*)(time1(n), n = 1, nend)
  close(100)

  do n = 1, nend
     read(time1(n),'(f4.0)')time2(n)
  enddo

  do n = 1, nend
     read(fln(n),'(a)')fln2(n)
  enddo

end subroutine read_fln



subroutine read_array
  use mndr  

  open(100,file = '../01_input_csv/'//trim(case)//'/'//trim(fln(n)))
  read(100,*)
  read(100,*)iend, jend
  close(100)

end subroutine read_array



subroutine cal_critical_shear_stress
  use mndr
  real*8 :: ustrc

  if(dm*100 .ge. 0.303)then
     ustrc = sqrt(80.9*(dm*100)) / 100
     tauc  = ustrc**2 / (s * g * dm)
  else if(dm*100 .ge. 0.118 .and. dm*100 .lt. 0.303)then
     ustrc = sqrt(134.6 * (dm*100)**(31./32)) / 100
     tauc  = ustrc**2 / (s * g * dm)
  else if(dm*100 .ge. 0.0565 .and. dm*100 .lt. 0.118)then
     ustrc = sqrt(55.0 * (dm*100)) / 100
     tauc  = ustrc**2 / (s * g * dm)
  else if(dm*100 .ge. 0.0065 .and. dm*100 .lt. 0.0565)then
     ustrc = sqrt(8.41 * (dm*100)**(11./32)) / 100
     tauc  = ustrc**2 / (s * g * dm)
  else if(dm*100 .lt. 0.0065)then
     ustrc = sqrt(226 * (dm*100)) / 100
     tauc  = ustrc**2 / (s * g * dm)
  endif

end subroutine cal_critical_shear_stress



subroutine read_scalar_csv
  use mndr

  open(100, file='../01_Input_csv/'//trim(case)//'/'//trim(fln(n)))

  read(100,'(a)')(chr, i = 1, 3)

  do j = 1, jend
     do i = 1, iend
        read(100,*)chr, chr, x(i,j), y(i,j), bl(i,j), nb(i,j), dep_cal(i,j), u_cal(i,j), v_cal(i,j)
     enddo
  enddo

  close(100)

  vel_cal(:,:)  = sqrt(u_cal(:,:)**2 + v_cal(:,:)**2)

end subroutine read_scalar_csv



subroutine calculate_spatial_slope(wl, slpi, slpj, slp)
  use mndr
  real*8,intent(in)  :: wl(iend,jend)
  real*8,intent(out) :: slpi(iend,jend), slpj(iend,jend), slp(iend,jend)

  do j = 1, jend
     do i = 1, iend
        if(i.eq.1)then
           slpi(i,j) = ( wl(i,j)   - wl(i+1,j) ) / ( x(i,j)   - x(i+1,j) )
        else if(i.eq.iend)then
           slpi(i,j) = ( wl(i,j)   - wl(i-1,j) ) / ( x(i,j)   - x(i-1,j) )
        else
           slpi(i,j) = ( wl(i-1,j) - wl(i+1,j) ) / ( x(i-1,j) - x(i+1,j) )
        endif
     enddo
  enddo

  do j = 1, jend
     do i = 1, iend
        if(j.eq.1)then
           slpj(i,j) = ( wl(i,j)   - wl(i,j+1) ) / ( y(i,j)   - y(i,j+1) )
        else if(j.eq.jend)then
           slpj(i,j) = ( wl(i,j)   - wl(i,j-1) ) / ( y(i,j)   - y(i,j-1) )
        else
           slpj(i,j) = ( wl(i,j-1) - wl(i,j+1) ) / ( y(i,j-1) - y(i,j+1) )
        endif
     enddo
  enddo

  do j = 1, jend
     do i = 1, iend
        slp(i,j) = sqrt(slpi(i,j)**2 + slpj(i,j)**2)
     enddo
  enddo

end subroutine calculate_spatial_slope



subroutine calculate_spatial_curvature(wl, curi, curj, cur)
  use mndr
  real*8,intent(in)  :: wl(iend,jend)
  real*8,intent(out) :: curi(iend,jend), curj(iend,jend), cur(iend,jend)

  do j = 1, jend
     do i = 1, iend
        if(i.eq.1)then
           curi(i,j) = ( (wl(i+2,j) - wl(i+1,j)) / (x(i+2,j) - x(i+1,j)) - (wl(i+1,j) - wl(i,j)) / (x(i+1,j) - x(i,j)) ) &
                * ( (x(i+1,j) - x(i,j)) / 2.0 )
        else if(i.eq.iend)then
           curi(i,j) = ( (wl(i,j) - wl(i-1,j)) / (x(i,j) - x(i-1,j)) - (wl(i-1,j) - wl(i-2,j)) / (x(i-1,j) - x(i-2,j)) ) &
                * ( (x(i,j) - x(i-2,j)) / 2.0 )
        else
           curi(i,j) = ( (wl(i+1,j) - wl(i,j)) / (x(i+1,j) - x(i,j)) - (wl(i,j) - wl(i-1,j)) / (x(i,j) - x(i-1,j)) ) &
                * ( (x(i+1,j) - x(i-1,j)) / 2.0 )
        endif
     enddo
  enddo

  do j = 1, jend
     do i = 1, iend
        if(j.eq.1)then
           curj(i,j) = ( (wl(i,j+2) - wl(i,j+1)) / (y(i,j+2) - y(i,j+1)) - (wl(i,j+1) - wl(i,j)) / (y(i,j+1) - y(i,j)) ) &
                * ( (y(i,j+1) - y(i,j)) / 2.0 )
        else if(j.eq.jend)then
           curj(i,j) = ( (wl(i,j) - wl(i,j-1)) / (y(i,j) - y(i,j-1)) - (wl(i,j-1) - wl(i,j-2)) / (y(i,j-1) - y(i,j-2)) ) &
                * ( (y(i,j) - y(i,j-2)) / 2.0 )
        else
           curj(i,j) = ( (wl(i,j+1) - wl(i,j)) / (y(i,j+1) - y(i,j)) - (wl(i,j) - wl(i,j-1)) / (y(i,j) - y(i,j-1)) ) &
                * ( (y(i,j+1) - y(i,j-1)) / 2.0 )
        endif
     enddo
  enddo

  do j = 1, jend
     do i = 1, iend
        cur(i,j) = sqrt(curi(i,j)**2 + curj(i,j)**2)
     enddo
  enddo

end subroutine calculate_spatial_curvature



subroutine calculate_energy_slope(rough, dep, veli, velj, slpi, slpj, slp)
  use mndr
  real*8              :: vel(iend,jend)
  real*8, intent(in)  :: rough(iend,jend), dep(iend,jend), veli(iend,jend), velj(iend,jend)
  real*8, intent(out) :: slpi(iend,jend), slpj(iend,jend), slp(iend,jend)

  do j = 1, jend
     do i = 1, iend
        vel(i,j)  = sqrt(veli(i,j)**2 + velj(i,j)**2)
        slp(i,j)  = ( rough(i,j) * vel(i,j) / dep(i,j)**(2./3.) )**2
        slpi(i,j) = slp(i, j) * cos( atan(velj(i, j) / veli(i, j)) )
        slpj(i,j) = slp(i, j) * sin( atan(velj(i, j) / veli(i, j)) )
        if(isnan(slpi(i,j)) .or. isnan(slpi(i,j)) .or. isnan(slpi(i,j)))then
           slpi(i,j) = 0.0
           slpj(i,j) = 0.0
           slp(i,j)  = 0.0
        endif
     enddo
  enddo

end subroutine calculate_energy_slope



subroutine calculate_advection_diffusion_equation_of_bed_level(nb1, h, &
     slp1_i , slp1_j , slp1, &
     slp2_i , slp2_j ,       &
     cb2_i  , cb2_j  ,       &
     u      , v      , vel,  &
     diexdxo, diexdyo,       &
     dieydxo, dieydyo,       &
     diedxo , diedyo ,       &
     dudxo  , dudyo  ,       &
     dvdxo  , dvdyo  ,       &
     dveldxo, dveldyo,       &
     Mxo    , Myo    , Mxyo, &
     fxo    , fyo    ,       &
     fx2o   , fy2o   ,       &
     Dfo    , dzo)
  use mndr
  real*8,intent(in)  :: nb1(iend,jend)    , h(iend,jend)
  real*8,intent(in)  :: slp1_i(iend,jend) , slp1_j(iend,jend), slp1(iend,jend)
  real*8,intent(in)  :: slp2_i(iend,jend) , slp2_j(iend,jend)
  real*8,intent(in)  :: cb2_i(iend,jend)  , cb2_j(iend,jend)
  real*8,intent(in)  :: u(iend,jend)      , v(iend,jend)     , vel(iend,jend)
  real*8,intent(in)  :: diexdxo(iend,jend), diexdyo(iend,jend)
  real*8,intent(in)  :: dieydxo(iend,jend), dieydyo(iend,jend)
  real*8,intent(in)  :: diedxo(iend,jend) , diedyo(iend,jend)
  real*8,intent(in)  :: dudxo(iend,jend)  , dudyo(iend,jend)
  real*8,intent(in)  :: dvdxo(iend,jend)  , dvdyo(iend,jend)
  real*8,intent(in)  :: dveldxo(iend,jend), dveldyo(iend,jend)
  real*8,intent(out) :: Mxo(iend,jend)    , Myo(iend,jend)   , Mxyo(iend,jend)
  real*8,intent(out) :: dzo(iend,jend)
  real*8,intent(out) :: fxo(iend,jend)    , fyo(iend,jend)
  real*8,intent(out) :: fx2o(iend,jend)   , fy2o(iend,jend)
  real*8,intent(out) :: Dfo(iend,jend)
  real*8             :: tau , mus, muk, gamma
  real*8             :: fx_1 , fx_2, fx_3, fx_4, fx_5, fx_6
  real*8             :: fy_1 , fy_2, fy_3, fy_4, fy_5, fy_6
  real*4             :: sig1, sig2

  do j = 1,jend
     do i = 1, iend

        if(h(i,j) .eq. 0.0)then
           tau   = 0.0
        else
           tau   = nb1(i,j)**2 * sqrt(u(i,j)**2 + v(i,j)**2 )**2 / (s * dm * h(i,j)**(1./3.))
        endif

        mus   = 0.4
        muk   = 0.8        
        gamma = sqrt(tauc/(mus*muk))

        ! ---- calculate Mx and My ----         
        Mxo(i,j)  = 4.*(tau - tauc)**(1./2.)*sqrt(s*g*dm**3) * slp1(i,j) / (s*dm*(1-0.4)) * &
             (u(i,j) / vel(i,j) - gamma / tau**(1./2.) * (1 - 1./(3*tau) * (tau - tauc) ) * ( slp2_i(i,j) ) )
        Myo(i,j)  = 4.*(tau - tauc)**(1./2.)*sqrt(s*g*dm**3) * slp1(i,j) / (s*dm*(1-0.4)) * &
             (v(i,j) / vel(i,j) - gamma / tau**(1./2.) * (1 - 1./(3*tau) * (tau - tauc) ) * ( slp2_j(i,j) ) )

        if((tau - tauc) .le. 0.0)then
           Mxo(i,j) = 0.0
           Myo(i,j) = 0.0
        endif
        if(vel(i,j) .eq. 0.0)then
           Mxo(i,j) = 0.0
           Myo(i,j) = 0.0
        endif

        ! ---- calculate M ----
        Mxyo(i,j) = sqrt( Mxo(i,j)**2 + Myo(i,j)**2 )


        ! ---- calculate fx_1 to fx_6 for fx ----
        if(slp1_i(i,j).eq.0.0)then
           fx_1 = 0.0
           fx_5 = 0.0
        else
           fx_1 =  3./5. * u(i,j)**2 / (g*slp1_i(i,j)) * diexdxo(i,j)
           fx_5 = u(i,j) * v(i,j) / (g*slp1_i(i,j)) * diexdyo(i,j)
        endif

        if(slp1_j(i,j).eq.0.0)then
           fx_3 = 0.0
        else
           fx_3 = -2./5. * u(i,j)*v(i,j) / (g*slp1_j(i,j)) * dieydyo(i,j)
        endif

        if(slp1(i,j).eq.0.0)then
           fx_2 = 0.0
           fx_4 = 0.0
        else
           fx_2 = -3./10. *       u(i,j)**2 / (g*slp1(i,j)) * diedxo(i,j) 
           fx_4 = -3./10. * u(i,j) * v(i,j) / (g*slp1(i,j)) * diedyo(i,j) 
        endif

        if(vel(i,j).eq.0.0)then
           fx_6 = 0.0
        else
           fx_6 = 6.* h(i,j) / vel(i,j) * dveldxo(i,j)
        endif

        ! ---- calculate fy_1 to fy_6 for fy ----
        if(slp1_j(i,j).eq.0.0)then
           fy_1 = 0.0
           fy_5 = 0.0
        else
           fy_1 =  3./5. * v(i,j)**2 / (g*slp1_j(i,j)) * dieydyo(i,j)
           fy_5 = u(i,j) * v(i,j) / (g*slp1_j(i,j)) * dieydxo(i,j)
        endif

        if(slp1_i(i,j).eq.0.0)then
           fy_3 = 0.0
        else
           fy_3 = -2./5. * u(i,j)*v(i,j) / (g*slp1_i(i,j)) * diexdxo(i,j)
        endif

        if(slp1(i,j).eq.0.0)then
           fy_2 = 0.0
           fy_4 = 0.0
        else
           fy_2 = -3./10. * v(i,j)**2 / (g*slp1(i,j)) * diedyo(i,j) 
           fy_4 = -3./10. * u(i,j) * v(i,j) / (g*slp1(i,j)) * diedxo(i,j) 
        endif

        if(vel(i,j).eq.0.0)then
           fy_6 = 0.0
        else
           fy_6 = 6.* h(i,j) / vel(i,j) * dveldyo(i,j)
        endif

        ! ---- calculate fx, fy ----        
        fxo(i,j) = fx_1 + fx_2 + fx_3 + fx_4 + fx_5 + fx_6
        fyo(i,j) = fy_1 + fy_2 + fy_3 + fy_4 + fy_5 + fy_6
        
        ! ---- calculate fx2, fy2 ----
        if((tau - tauc) .le. 0.0)then
           fx2o(i,j) = 0.0
           fy2o(i,j) = 0.0
        else
           if(vel(i,j).eq.0.0)then
              fx2o(i,j) = 0.0
              fy2o(i,j) = 0.0
           else
              fx2o(i,j) = 8.*(tau - tauc)**(3./2.)*sqrt(s*g*dm**3) / (1-0.4) * &
                   ( 1./vel(i,j)*dudxo(i,j) - u(i,j)/(vel(i,j)**(2.))*dveldxo(i,j) )
              fy2o(i,j) = 8.*(tau - tauc)**(3./2.)*sqrt(s*g*dm**3) / (1-0.4) * &
                   ( 1./vel(i,j)*dvdyo(i,j) - v(i,j)/(vel(i,j)**(2.))*dveldyo(i,j) )
           endif
        endif
        
        ! ---- calculate Diffusion coefficient ----
        if((tau - tauc) .le. 0.0)then
           Dfo(i,j) = 0.0
        else
           Dfo(i,j) = 8.*(tau - tauc)**(3./2.)*sqrt(s*g*dm**3) / (1-0.4) * (gamma / tau**(1./2.))
        endif

        ! ---- calculate dz ----
        dzo(i,j) = ( &
             - Mxo(i,j) * (slp2_i(i,j) + slp1_i(i,j) + fxo(i,j)) &
             - Myo(i,j) * (slp2_j(i,j) + slp1_j(i,j) + fyo(i,j)) &
             + Dfo(i,j) * cb2_i(i,j) &
             + Dfo(i,j) * cb2_j(i,j) &
             - fx2o(i,j) &
             - fy2o(i,j) &
             ) * dt

     enddo
  enddo
  
end subroutine calculate_advection_diffusion_equation_of_bed_level



subroutine output_scalar_result
  use mndr

  call system('mkdir -p ../03_output_result')
  call system('mkdir -p ../03_output_result/'//trim(case))

  open(100,file='../03_output_result/'//trim(case)//'/'//trim(fln(n)(1:len(trim(fln(n)))-4))//'.csv')
  write(100,*)'time[min]=,',fln2(n)
  write(100,*)iend,',', jend
  write(100,*)&
       'i,'                              , &
       'j,'                              , &
       'x,'                              , &
       'y,'                              , &
       'bed level [m],'                  , &
       'Water depth [m],'                , &
       'Velocity (Long.) [m/s],'         , &
       'Velocity (Cros.) [m/s}],'        , &
       'Velocity (Mag) [m/s}],'          , &
       'Migrating speed (Long.) [m/s],'  , &
       'Migrating speed (Cros.) [m/s],'  , &
       'Migrating speed (Mag.)  [m/s],'  , &
       'Riverbed variation [m],'

  do j = 1, jend
     do i = 1, iend
        write(100,*)&
             i                  , ',', &
             j                  , ',', &
             x(i,j)             , ',', &
             y(i,j)             , ',', &
             bl(i,j)            , ',', &
             dep_cal(i,j)       , ',', &
             u_cal(i,j)         , ',', &
             v_cal(i,j)         , ',', &
             vel_cal(i,j)       , ',', &
             Mx(i,j)            , ',', &
             My(i,j)            , ',', &
             Mxy(i,j)           , ',', &
             dz_cal(i,j)

     enddo
  enddo

  close(100)

end subroutine output_scalar_result
