#How to Use the Program of Calculating Migrating speed and Reverbed variation

This Readme describes a manual of program that calculates the migrating speed of alternate bars and riverbed variations using general hydraulic quantities obtained from measurements or calculations in two planar dimensions.

##Directory Structure
```
Refrerence_material/
  ├──01_Input_csv/
  |		└── 20210715(sample data)/
  | 		├── 0001_trm.csv
  |			├── 0040_trm.csv
  |			├── 0080_trm.csv
  |			└── 0120_trm.csv
  |	
  ├── 02_Analysis(Main Program)/
  |		├── f95/
  |		|		├── CmnVal_vis_vtk.f95
  |		|		└── main.f95
  |		├── file.txt
  |		├── mndr.mod
  |		├── Makefile
  |		└── out.o
  |		
  └── 03_output_result/
  		└── 20210715(sample data)/
  	 		├── 0001_trm.csv
  			├── 0040_trm.csv
  			├── 0080_trm.csv
  			└── 0120_trm.csv

```

##How to run the program
####Step 1：Add the data you want to analyze to 01_Input_csv/
Using the sample data csv file as a reference, create the data you wish to analyze and add it so that it becomes a date/csv file.

####Step 2：Change variable "case", "dm", "dt" in 02_Analysis/f95/main.f95 to the directory name of the data to be analyzed.

The contents of 02_Analysis/f95/main.f95 are shown below.

```
program main
  use mndr

...
<Omitted>.
...

subroutine set_parameter
  use mndr

  case = '20210715'   !  You should select directory name in 01_Input_csv
  dm   = 0.00076      !  average diameter of soil particles [m]
  dt   = 60           !  Delta t for calculation of Riverbed variation [s]

  pi   = acos(-1.0d0) !  pi
  g    = 9.81         !  gravitational accelaration [m/s^2]
  s    = 1.65         !  underwater specific gravity of soil particles

end subroutine set_parameter
```


####Step 3：In the terminal, type "make && ./out.o".
Check the contents of the Makefile and specify the version of Fortran you wish to run. The version I am currently using is gcc11, so the contents of the Makefile are as follows.

```
Main       = f95/main
Cmnvl      = f95/CmnVal_vis_vtk
Out        = out
F95        = /opt/local/bin/gfortran-mp-11
SOURCE     = $(Cmnvl).f95 $(Main).f95

$(Out).o: $(SOURCE)
	$(F95) -o $@ $(SOURCE)
```

After specifying the Fortran version, go to 02_Analysis in the terminal and execute the command `make && . /out.o`.

After successful execution, the results are output to 03_output_result/.